/***************************************************************************
 *  Description:
 *      Listen for incoming connections and send back a string containing
 *      the client's IP address.  This allows the client to determine the
 *      outside address of any NAT gateways it's going through.
 *
 *  Arguments:
 *      1.  TCP port on which to listen
 *
 *  Returns:
 *      See sysexits.h
 *
 *  TODO:
 *      Log messages
 *      Terminate properly when service is stopped
 *      Support IPv6
 *      Protect against DOS attacks
 *
 *  History: 
 *  Date        Name        Modification
 *  2017-06-23  Jason Bacon Begin
 ***************************************************************************/

/* SunOS Compile: cc prog -lsocket */
/* SCO_SV Compile: cc -v prog -lsocket */

/* server.c */
#include <stdio.h>
#include <string.h>
#include <stdlib.h>
#include <unistd.h>
/* Place sysexits after unistd to work around a bug in IRIX.  EX_OK
   is defined differently in these two headers, and we want the
   sysexists version to override here */
#include <sysexits.h>
#include <sys/socket.h>
#include <netinet/in.h>
#include <arpa/inet.h>

#define MAX_QUEUE       10
#define IP_ADDR_SIZE    256

int     main (int argc, char *argv[])
{
    int                 connect_fd, listen_fd;
    struct sockaddr_in  server_address;
    struct sockaddr_in  client_address;
    socklen_t           address_len = sizeof (client_address);
    short               tcp_port;   /* Need short for htons() */
    uint32_t            addr;
    char                ip_addr[IP_ADDR_SIZE + 1];
    extern int          errno;

    if (argc != 2)
    {
	fprintf (stderr, "Usage: %s <port>\n", argv[0]);
	return EX_USAGE;
    }
    
    daemon(0,0);

    /* Get port number from command line */
    tcp_port = atoi (argv[1]);
    if ((listen_fd = socket (AF_INET, SOCK_STREAM, 0)) < 0)
    {
	fprintf (stderr, "Error opening socket \n");
	exit (1);
    }

    server_address.sin_family = AF_INET;
    server_address.sin_addr.s_addr = htonl (INADDR_ANY);
    server_address.sin_port = htons (tcp_port);

    /* Bind socket to server address */
    if (bind (listen_fd, (struct sockaddr *) &server_address,
	      sizeof (server_address)) < 0)
    {
	perror ("bind() failed");
	return 1;
    }

    /* Listen for connection requests */
    if (listen (listen_fd, MAX_QUEUE) != 0)
    {
	fputs ("listen() failed.\n", stderr);
	return 1;
    }

    while ( 1 )
    {
	/* Accept a connection request */
	if ((connect_fd = accept(listen_fd, (struct sockaddr *)&client_address,
		    &address_len)) == -1)
	{
	    fputs ("accept() failed.\n", stderr);
	    perror("reflectipd");
	    break;
	}
	addr=client_address.sin_addr.s_addr;
	snprintf(ip_addr, IP_ADDR_SIZE, "%u.%u.%u.%u",
	    addr & 0xff, (addr >> 8 ) & 0xff,
	    (addr >> 16 ) & 0xff, (addr >> 24 ) & 0xff);
	write (connect_fd, ip_addr, strlen(ip_addr)+1);
	close (connect_fd);
    }
    
    return EX_OK;
}
